import configparser
import os
import re
from copy import deepcopy


class Config:

    def __init__(self, value_dict):
        self.values = value_dict

    def set(self, name, value):
        self.values[name] = value

    def get(self, name, default=None, as_type=None):
        val = self.values.get(name) or default
        return as_type(val) if as_type else val

    def get_bool(self, name, default=False):
        val = self.get(name, default)
        return str(val).lower() == 'true'

    def get_int(self, name, default=None):
        return int(self.get(name, default))

    def get_float(self, name, default=None):
        return float(self.get(name, default))

    def get_list(self, name, default=None):
        return list(self.get(name, default))

    def get_dict(self, name, default=None):
        return dict(self.get(name, default))

    def get_path(self, name, default=None):
        raw = self.get(name, default)
        if not raw:
            return None
        raw = re.split('[/\\\\]+', raw)
        return os.sep.join(raw)


class ConfigLoader:

    def __init__(self, props_file):
        self.values = self.from_properties(props_file)

    @classmethod
    def load(cls, props_file):
        return cls(props_file)

    def get_config(self, name):
        if name in self.values:
            return Config(self.get_dict_config(name))

    def get_dict_config(self, name=None):
        if not name:
            return deepcopy(self.values)
        else:
            return deepcopy(self.values.get(name))

    def from_properties(self, props_file):
        config = {}
        parser = configparser.RawConfigParser()
        parser.read(props_file)

        for s, v in parser._sections.items():
            config[s] = v
        return config

    def set(self, section, name, value):
        if section not in self.values:
            self.values[section] = {}
        self.values[section][name] = value

    def get(self, section, name, default=None, as_type=None):
        if section not in self.values:
            val = default
        else:
            val = self.values[section].get(name) or default
        return as_type(val) if as_type else val

    def get_bool(self, section, name, default=False):
        val = self.get(section, name, default)
        return str(val).lower() == 'true'

    def get_int(self, section, name, default=None):
        return int(self.get(section, name, default))

    def get_list(self, section, name, default=None):
        return list(self.get(section, name, default))

    def get_dict(self, section, name, default=None):
        return dict(self.get(section, name, default))

    def get_path(self, section, name, default=None):
        raw = self.get(section, name, default)
        if not raw:
            return None
        raw = re.split('[/\\\\]+', raw)
        return os.sep.join(raw)



import base64
import json
import time

import requests
from PyQt5.QtCore import QThread, pyqtSlot, QSize, pyqtSignal
from PyQt5.QtGui import QPixmap
from PyQt5.QtWidgets import QLabel, QSizePolicy


class PiCamGenerator(QThread):
    updateList = pyqtSignal(object)
    updateCameras = pyqtSignal(object)

    def __init__(self, config, parent=None):
        super(PiCamGenerator, self).__init__(parent)
        self.list_url = config.get('list_url')
        self.data_url = config.get('data_url')
        self.sleep = config.get_float('update_interval', 0.1)
        self.list_refresh = config.get_float('list_refresh', 10)

    def getCameraList(self):
        cams = requests.get(self.list_url).content
        return json.loads(cams)

    def update(self):
        img = requests.get(self.data_url)
        data = json.loads(img.content)
        camData = {v['source']: v for v in data}
        self.updateCameras.emit(camData)

    def run(self):
        start = time.time()
        self.updateList.emit(self.getCameraList())
        while True:
            self.update()
            time.sleep(self.sleep)
            if time.time() - start > self.list_refresh:
                self.updateList.emit(self.getCameraList())
                start = time.time()


class Camera(QLabel):
    def __init__(self, source=None, size=300, name="default", scaled=False, parent=None):
        super(Camera, self).__init__(parent)
        self.setSizePolicy(QSizePolicy.Expanding, QSizePolicy.Expanding)
        self.setMinimumSize(QSize(50, 50))
        self.px = None
        self.name = name
        self.size = size
        self.setScaledContents(scaled)
        source.updateCameras.connect(self.setImage)

    @pyqtSlot(object, name="size")
    def setFrameSize(self, size):
        if self.px:
            self.setPixmap(self.px.scaled(size, size))
            self.size = size

    @pyqtSlot(object, name="setimage")
    def setImage(self, camData=None):
        if self.name in camData:
            img = self.getImage(camData[self.name]['image'])
            self.px = QPixmap()
            self.px.loadFromData(img)
            self.setFrameSize(self.size)

    def getImage(self, data):
        byte = data.encode('utf-8')
        return base64.decodebytes(byte)



import math


def computeGrid(camCount, width, height):

    cols, rows, sideLength = calculateCols(camCount, width, height)
    remainder_height = height - sideLength * rows

    if remainder_height < 0:
        sideLength = sideLength - abs(remainder_height / rows)

    rheight = max(height - sideLength * rows, 0) / 2
    vheight = max(width - sideLength * cols, 0) / 2
    dimensions = (vheight, rheight, vheight, rheight)
    return cols, dimensions, sideLength


def calculateCols(camCount, width, height):
    """
    # Iterative computation to determine actual cols

    """
    # Start by initial guess that ncols = ncams
    cols = rows = sideLength = camCount
    while (cols > 0):
        rows = math.ceil(camCount / cols)
        sideLength = width / cols
        if ((height - sideLength * rows) < sideLength
                and (rows*cols - camCount) <= 1):
            break
        cols -= 1
    cols = 1 if cols < 1 else cols
    return cols, rows, sideLength

def areSetsEqual(a, b):
    a = a or set()
    b = b or set()
    a = set(a)
    b = set(b)
    return  (a - b) == (b - a)
